MODULE p4zsink
   !!======================================================================
   !!                         ***  MODULE p4zsink  ***
   !! TOP :   PISCES Compute vertical flux of particulate matter due to gravitational sinking
   !!======================================================================
   !! History :   1.0  !  2004     (O. Aumont) Original code
   !!             2.0  !  2007-12  (C. Ethe, G. Madec)  F90
   !!----------------------------------------------------------------------
#if defined key_pisces  && ! defined key_kriest
   !!----------------------------------------------------------------------
   !!   'key_pisces'        and                            PISCES bio-model
   !!   NOT 'key_kriest'                                   No Kriest option
   !!----------------------------------------------------------------------
   !!   p4z_sink       :  Compute vertical flux of particulate matter due to gravitational sinking
   !!----------------------------------------------------------------------
   USE oce_trc         !
   USE trp_trc
   USE sms
   USE p4zsink2        !

   IMPLICIT NONE
   PRIVATE

   PUBLIC   p4z_sink    ! called in p4zbio.F90

   !!* Substitution
#  include "domzgr_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/TOP 2.0 , LOCEAN-IPSL (2007) 
   !! $Header:$ 
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE p4z_sink
      !!---------------------------------------------------------------------
      !!                     ***  ROUTINE p4z_sink  ***
      !!
      !! ** Purpose :   Compute vertical flux of particulate matter due to 
      !!              gravitational sinking
      !!
      !! ** Method  : - ???
      !!---------------------------------------------------------------------
      INTEGER  ::   ji, jj, jk
      INTEGER  ::   iksed
      REAL(wp) ::   zagg1, zagg2, zagg3, zagg4
      REAL(wp) ::   zfact, zstep, zwsmax
#if defined key_trc_dia3d
      REAL(wp) ::   zrfact2
#endif
      REAL(wp), DIMENSION(jpi,jpj,jpk) ::   zsinking, zsinking2
      REAL(wp), DIMENSION(jpi,jpj,jpk) ::   zsinkfer, zsinkfer2
      REAL(wp), DIMENSION(jpi,jpj,jpk) ::   zsinkcal, zsinksil
      !!---------------------------------------------------------------------

       zstep = rfact2 / rjjss      ! Timestep duration for biology


!    Sinking speeds of detritus is increased with depth as shown
!    by data and from the coagulation theory
!    -----------------------------------------------------------

      iksed = 10

      DO jk = 1, jpkm1
         DO jj = 1, jpj
            DO ji=1,jpi
               zfact = MAX( 0., fsdepw(ji,jj,jk+1)-hmld(ji,jj) ) / 4000.
               wsbio4(ji,jj,jk) = wsbio2 + ( 200.- wsbio2 ) * zfact
            END DO
         END DO
      END DO

!      LIMIT THE VALUES OF THE SINKING SPEEDS 
!      TO AVOID NUMERICAL INSTABILITIES

      wsbio3(:,:,:) = wsbio

      DO jk = 1,jpkm1
         DO jj = 1, jpj
            DO ji = 1, jpi
               zwsmax = 0.8 * fse3t(ji,jj,jk) / zstep
               wsbio4(ji,jj,jk) = MIN( wsbio4(ji,jj,jk), zwsmax )
               wsbio3(ji,jj,jk) = MIN( wsbio3(ji,jj,jk), zwsmax )
            END DO
         END DO
      END DO

      wscal(:,:,:) = wsbio4(:,:,:)


!   INITIALIZE TO ZERO ALL THE SINKING ARRAYS
!   -----------------------------------------

      zsinking (:,:,:) = 0.e0
      zsinking2(:,:,:) = 0.e0
      zsinkcal (:,:,:) = 0.e0
      zsinkfer (:,:,:) = 0.e0
      zsinksil (:,:,:) = 0.e0
      zsinkfer2(:,:,:) = 0.e0

!   Compute the sedimentation term using p4zsink2 for all
!   the sinking particles
!   -----------------------------------------------------

      CALL p4z_sink2( wsbio3, zsinking , jppoc )
      CALL p4z_sink2( wsbio3, zsinkfer , jpsfe )
      CALL p4z_sink2( wsbio4, zsinking2, jpgoc )
      CALL p4z_sink2( wsbio4, zsinkfer2, jpbfe )
      CALL p4z_sink2( wsbio4, zsinksil , jpdsi )
      CALL p4z_sink2( wscal , zsinkcal , jpcal )

!  Exchange between organic matter compartments due to
!  coagulation/disaggregation
!  ---------------------------------------------------

      DO jk = 1, jpkm1
         DO jj = 1, jpj
            DO ji = 1, jpi

               zfact = zstep * zdiss(ji,jj,jk)

!    Part I : Coagulation dependent on turbulence
!    ----------------------------------------------

# if defined key_off_degrad
               zagg1 = 940.* zfact * trn(ji,jj,jk,jppoc) * trn(ji,jj,jk,jppoc) * facvol(ji,jj,jk)
# else
               zagg1 = 940.* zfact * trn(ji,jj,jk,jppoc) * trn(ji,jj,jk,jppoc)
# endif

# if defined key_off_degrad
               zagg2 = 1.054e4 * zfact * trn(ji,jj,jk,jppoc) * trn(ji,jj,jk,jpgoc) * facvol(ji,jj,jk)
# else
               zagg2 = 1.054e4 * zfact * trn(ji,jj,jk,jppoc) * trn(ji,jj,jk,jpgoc)
# endif

!    Aggregation of small into large particles
!    Part II : Differential settling
!    ----------------------------------------------

# if defined key_off_degrad
               zagg3 = 0.66 * zstep * trn(ji,jj,jk,jppoc) * trn(ji,jj,jk,jpgoc) * facvol(ji,jj,jk)
# else
               zagg3 = 0.66 * zstep * trn(ji,jj,jk,jppoc) * trn(ji,jj,jk,jpgoc)
# endif

# if defined key_off_degrad
!!gm  zagg4 set to zero ????
               zagg4 = 0.e0 * zstep * trn(ji,jj,jk,jppoc) * trn(ji,jj,jk,jppoc) * facvol(ji,jj,jk)
# else
!!gm  zagg4 set to zero ????
               zagg4 = 0.e0 * zstep * trn(ji,jj,jk,jppoc) * trn(ji,jj,jk,jppoc)
# endif

               xagg  (ji,jj,jk) = zagg1 + zagg2 + zagg3 + zagg4
               xaggfe(ji,jj,jk) = xagg(ji,jj,jk) * trn(ji,jj,jk,jpsfe) / ( trn(ji,jj,jk,jppoc) + rtrn )

!     Aggregation of DOC to small particles
!     --------------------------------------

               xaggdoc(ji,jj,jk) = ( 80.* trn(ji,jj,jk,jpdoc) + 698. * trn(ji,jj,jk,jppoc) )       &
# if defined key_off_degrad
                  &              * facvol(ji,jj,jk)                           &
# endif
                  &              * zfact * trn(ji,jj,jk,jpdoc)

               xaggdoc2(ji,jj,jk) = 1.05e4 * zfact * trn(ji,jj,jk,jpgoc)   &
# if defined key_off_degrad
                  &               * facvol(ji,jj,jk)                            &
# endif      
                  &               * trn(ji,jj,jk,jpdoc)

            END DO
         END DO
      END DO

# if defined key_trc_dia3d
      zrfact2 = 1.e3 * rfact2r
      trc2d(:,:, 5) = zsinking (:,:,iksed+1) * zrfact2
      trc2d(:,:, 6) = zsinking2(:,:,iksed+1) * zrfact2
      trc2d(:,:, 7) = zsinkfer (:,:,iksed+1) * zrfact2
      trc2d(:,:, 8) = zsinkfer2(:,:,iksed+1) * zrfact2
      trc2d(:,:, 9) = zsinksil (:,:,iksed+1) * zrfact2
      trc2d(:,:,10) = zsinkcal (:,:,iksed+1) * zrfact2
# endif
      !
   END SUBROUTINE p4z_sink

#else
   !!======================================================================
   !!  Dummy module :                                   No PISCES bio-model
   !!======================================================================
CONTAINS
   SUBROUTINE p4z_sink                    ! Empty routine
   END SUBROUTINE p4z_sink
#endif 

   !!======================================================================
END MODULE  p4zsink
